function [Pre_Labels,Outputs,Accuracy]= M3PL( train_data,train_p_target,test_data,test_target,C_max)

%    main function
%    Syntax
%
%       [Pre_Labels,Outputs,Accuracy]= M3PL( train_data,train_p_target,test_data,test_target,C)
%
%    Description
%
%       PLL_MIL takes,
%           train_data     - An MxN array, the ith instance of training instance is stored in train_data(i,:)
%           train_p_target - A QxM array, if the jth class label is one of the partial labels for the ith training instance, then train_p_target(j,i) equals +1, otherwise train_p_target(j,i) equals 0
%           test_data      - An M2xN array, the ith instance of testing instance is stored in test_data(i,:)
%           test_target    - A QxM2 array, if the jth class label is the ground-truth label for the ith test instance, then test_target(j,i) equals 1; otherwise test_target(j,i) equals 0
%           C_max          - The maximum value for regularization parameter
%      and returns,
%           Pre_Labels     - A QxM2 array, if the ith test instance is predicted to have the jth class label, then Pre_Labels(j,i) is 1, otherwise Pre_Labels(j,i) is 0
%           Outputs        - A QxM2 array, the numerical output of the ith test instance on the jth class label is stored in Outputs(j,i)
%           Accuracy       - Predictive accuracy on the test set

train_p_target = full(train_p_target);
 ins_num = size(train_data,1);
    label_num = size(train_p_target,1);
    fea_num = size(train_data,2);
    confident = train_p_target./repmat(sum(train_p_target),label_num,1);
    sumConfident = full(sum(confident,2));
    labelNumVec = floor(sumConfident);
    sorted = sortrows([sumConfident,(1:label_num)']);
    for itemLabel = 1:ins_num-sum(labelNumVec)
        labelNumVec(round(sorted(itemLabel,2))) = labelNumVec(round(sorted(itemLabel,2))) + 1;
    end
   initLabel = assignLabel(-confident,train_p_target,labelNumVec);
    sorted = sortrows([initLabel,train_data,full(train_p_target)']);
    initLabel = sorted(:,1);
    train_data = sorted(:,2:fea_num+1);
    train_p_target = sorted(:,fea_num+2:end)';
    ctmp = (1e-5)*C_max;
 
    flag = 0;
    while 1
        if ctmp==C_max
            flag = 1;
        end
        str = horzcat('-s 4 -B 1 -q -c ','',num2str(ctmp));
        disp(['the value of C:',num2str(ctmp)]);
        lastValue = 0;
        for iter = 1:10
            model = train(initLabel,sparse(train_data),str);
            decision_value = model.w*[train_data,ones(ins_num,1)]';
            tmp_target = accumarray([initLabel,(1:ins_num)'],ones(ins_num,1));
            [Pre_Labels,Accuracy,Outputs] = predict(initLabel,sparse(train_data),model,'-q');
%             pred
            w = model.w;
            modelValue = getModelValue(tmp_target,decision_value,w,ctmp);
            if abs(modelValue-lastValue)<1e-4
                break
            end
            lastValue = modelValue;
            margin = zeros(label_num,ins_num);
            for i=1:label_num
                tmpLabel = zeros(label_num,ins_num);
                tmpLabel(i,:) = ones(1,ins_num);
                maxOut = max(Outputs'.*~tmpLabel);
                margin(i,:) = max(0,1-(Outputs(:,i)'-maxOut));
            end
            initLabel = assignLabel(-margin,train_p_target,labelNumVec);
            sorted = sortrows([initLabel,train_data,full(train_p_target)']);
            initLabel = sorted(:,1);
            train_data = sorted(:,2:fea_num+1);
            train_p_target = sorted(:,fea_num+2:end)';
        end
        if flag==1
            break
        end
        ctmp = ctmp*1.5;
         if ctmp>C_max
            ctmp = C_max;
        end

    end
    [~,test_label] = max(full(test_target));
    test_num = size(test_data,1);
    [Pre_Labels,acci,Outputs] = predict(test_label',sparse(test_data),model);
    pred = zeros(label_num,test_num);
    pred(repmat((1:label_num)',1,test_num)==repmat(Pre_Labels',label_num,1))=1;
    Pre_Labels = pred;
Accuracy = acci(1);
end

